#ifndef LATTICE_POINT_CLASS_H
#define LATTICE_POINT_CLASS_H
#include <iostream>
#include "bubble.h"

//enum where {interior, surface, edge, vertex};

class simple_lattice_point {
 protected:
  bubble* bub;                      // The bubble it belongs to

 public:
  simple_lattice_point(bubble* bb = 0) : bub(bb) {}
  ~simple_lattice_point() {}

  // setting spin of lattice point, used during initialization
  void set_spin(bubble* bb) { bub = bb; }
  void set_spin(simple_lattice_point p) { bub = p.bub; }
  void update_sum_xy(int x, int y) { bub->update_sum_xy(x, y); }

  // overloaded assignment operator, used during coarsening
  simple_lattice_point operator=(const simple_lattice_point &p) {
    bub = p.bub; 
    return *this; }

  // overloaded << operator
  friend std::ostream& operator<<(std::ostream &strm, 
				  const simple_lattice_point &lp) {
    strm.width(2);
    strm << lp.get_spin();
    return strm;
  }

  // getting spins. simple_lattice_point is declared as a friend of bubble,
  // so it can directly access bubble's spin
  const int get_spin() const { return bub->spin; }
  bubble* get_bubble_pointer() const { return bub; }
  const int get_type() const { return bub->type; }
  const int get_area() const { return bub->area; }
  const int get_target_area() const { return bub->target_area; }
  const double get_bubble_energy() const { return bub->energy; }

  // comparing spins between two lattice points
  bool operator==(simple_lattice_point& p) {return bub->spin == (p.bub)->spin;}
  bool operator!=(simple_lattice_point& p) {return bub->spin != (p.bub)->spin;}

  // increase and decrease bubbble's volume
  void increase_bubble_area() { bub->increase_area(); }
  void decrease_bubble_area() { bub->decrease_area(); }
  void increment_bubble_energy(int de) { bub->increment_energy(de); }
};

// compound_lattice_point inherits from simple_lattice_point.
// compound_lattice_point also stores the neighbouring lattice points.
// The reason for not combining the two into a single class is to
// avoid infinite recursion when initializing lattice points :
// initializing a lattice point leads to initializing its neighbours,
// which then leads to initializing neighbours' neighbours, ad infinitum.
class compound_lattice_point : public simple_lattice_point {
 private:
  simple_lattice_point** neighbour;  // The neighbouring lattice points

 public:
  compound_lattice_point() { neighbour = new simple_lattice_point* [33]; }
  ~compound_lattice_point() { delete [] neighbour; }

  void set_neighbour(int n, simple_lattice_point* p) { *(neighbour + n) = p; }
  simple_lattice_point** list_of_neighbours() { return neighbour; } 
};

#endif
