#ifndef RAN3_FLOAT_H
#define RAN3_FLOAT_H

/**************************************************************/
/*                                                            */
/*           Lagged Fibonacci random number generator         */
/*           ----------------------------------------         */
/*                                                            */
/*              X_n = (X_(n-55) - X_(n-24)) mod m             */
/*                                                            */
/*                Documentation                               */
/*                -------------                               */
/*                                                            */
/* 30/8/2001    MBIG = 1000000000                             */
/* 17/9/2001    MBIG = (1L<<30)                               */
/* 17/9/2001    #define mod_diff(x,y) (((x)-(y))&(MBIG-1))    */
/*              ran3f() returns a number in the range [0, 1]  */             
/*                                                            */
/*                   Usage                                    */
/*                   -----                                    */
/*                                                            */
/*        Call with negative seed idum                        */
/*                                                            */
/**************************************************************/

#include <cstdlib>
#define MBIG (1L<<30)           // modulus
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)
#define SLAG 24                 // short lag
#define LLAG 55                 // long lag
#define mod_diff(x,y)   (((x)-(y))&(MBIG-1))  // (x-y)mod MBIG

float ran3f(long *idum)
{
	int i,ii,k;
	static int iff=0;
	static int inext,inextp;
	static long ma[LLAG+1];
        long mj,mk;

	if (*idum < 0 || iff == 0) {
		iff=1;
		mj=MSEED-(*idum < 0 ? -*idum : *idum);
		mj %= MBIG;
		ma[LLAG]=mj;
		mk=1;
		for (i=1;i<=(LLAG-1);i++) {
			ii=(21*i) % LLAG;
			ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=ma[ii];
		}
		for (k=1;k<=4;k++)
			for (i=1;i<=LLAG;i++) {
				ma[i] -= ma[1+(i+30) % LLAG];
				if (ma[i] < MZ) ma[i] += MBIG;
			}
		inext=0;
		inextp= (LLAG - SLAG);
		*idum=1;
	}
	if (++inext == (LLAG+1)) inext=1;
	if (++inextp == (LLAG+1)) inextp=1;
	mj = mod_diff(ma[inext], ma[inextp]);
	ma[inext]=mj;
	return mj*FAC;
}

#undef MBIG
#undef MSEED
#undef MZ
#undef FAC
#undef SLAG
#undef LLAG
#undef mod_diff

#endif
/* (C) Copr. 1986-92 Numerical Recipes Software W$5&+1*?$. */
