#ifndef RAN3_LONG_H
#define RAN3_LONG_H

/**************************************************************/
/*                                                            */
/*           Lagged Fibonacci random number generator         */
/*           ----------------------------------------         */
/*                                                            */
/*              X_n = (X_(n-55) - X_(n-24)) mod m             */
/*                                                            */
/*                Documentation                               */
/*                -------------                               */
/*                                                            */
/* 30/08/2001    MBIG = 1000000000                            */
/* 17/09/2001    MBIG = (1L<<30)                              */
/* 17/09/2001    #define mod_diff(x,y) (((x)-(y))&(MBIG-1))   */
/* 21/09/2001    change ran3l() so that it returns an integer */
/*               between 0 and MBIG-1 (1073741823)            */
/*                                                            */
/*                   Usage                                    */
/*                   -----                                    */
/*                                                            */
/*    Call with either positive or negative seed idum         */
/*                                                            */
/**************************************************************/

#define MBIG (1L<<30)        // modulus
#define MSEED 161803398
#define MZ 0
#define SLAG 24              // short lag
#define LLAG 55              // long lag
#define mod_diff(x,y)  (((x)-(y))&(MBIG-1))  // (x-y)mod MBIG

long ran3l(long *idum)
{
        int i, ii, k;
	static int initialize = 1;
	static int inext,inextp;
	static long ma[LLAG+1];
        long mj,mk;

	if (*idum < 0 || initialize) {
		initialize = 0;
		mj=MSEED-(*idum < 0 ? -*idum : *idum);
		mj %= MBIG;
		ma[LLAG]=mj;
		mk=1;
		for (i=1;i<=(LLAG-1);i++) {
			ii=(21*i) % LLAG;
			ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=ma[ii];
		}
		for (k=1;k<=4;k++)
			for (i=1;i<=LLAG;i++) {
				ma[i] -= ma[1+(i+30) % LLAG];
				if (ma[i] < MZ) ma[i] += MBIG;
			}
		inext = 0;
		inextp = (LLAG - SLAG);	
		*idum = 1;
	}
	if (++inext == (LLAG+1)) inext=1;
	if (++inextp == (LLAG+1)) inextp=1;
	mj = mod_diff(ma[inext], ma[inextp]);
	ma[inext]=mj;
	return mj;
}

#undef MBIG
#undef MSEED
#undef MZ
#undef SLAG
#undef LLAG
#undef mod_diff

#endif
/* (C) Copr. 1986-92 Numerical Recipes Software W$5&+1*?$. */
